from fastapi import FastAPI, Request, status
from fastapi.routing import APIRoute
from starlette.middleware.base import BaseHTTPMiddleware
from starlette.responses import JSONResponse
from urllib.parse import urlsplit


class SetRootPathMiddleware(BaseHTTPMiddleware):
    """
    Set `root_path` to make proper URLs behind a proxy
    https://fastapi.tiangolo.com/advanced/behind-a-proxy/
    """

    async def dispatch(self, request: Request, call_next):
        original_url = request.headers.get("x-envoy-original-path")
        path = request.url.path

        if original_url:
            original_path = urlsplit(original_url).path

            if original_path.endswith(path):
                request.scope["root_path"] = original_path.removesuffix(path)

        return await call_next(request)


class App(FastAPI):
    def setup(self) -> None:
        if not self.swagger_ui_init_oauth:
            self.swagger_ui_init_oauth = {"clientId": "delphai-api"}

        super().setup()

        self.add_middleware(SetRootPathMiddleware)
        self.add_event_handler("startup", self._include_dependency_responses)

        try:
            import httpx
        except ImportError:
            pass
        else:
            self.add_exception_handler(httpx.TimeoutException, self._timeout_error)

    def _include_dependency_responses(self):
        for route in self.routes:
            if isinstance(route, APIRoute):
                for dependency in self._walk_dependency_tree(route.dependant):
                    dependency_responses = getattr(dependency.call, "responses", None)
                    if dependency_responses:
                        route.responses = dict(dependency_responses, **route.responses)

                endpoint_responses = getattr(route.endpoint, "responses", None)
                if endpoint_responses:
                    route.responses = dict(endpoint_responses, **route.responses)

    def _walk_dependency_tree(self, dependant, visited=None):
        if visited is None:
            visited = set()
        visited.add(dependant.cache_key)

        for sub_dependant in dependant.dependencies:
            if sub_dependant.cache_key in visited:
                continue

            yield sub_dependant
            yield from self._walk_dependency_tree(sub_dependant, visited)

    def _timeout_error(self, request, error):
        return JSONResponse(
            status_code=status.HTTP_504_GATEWAY_TIMEOUT,
            content={"detail": "Gateway Timeout"},
        )



import httpx
import logging

from typing import Annotated, Any, Optional

from fastapi import Depends, HTTPException, Request, status
from fastapi.security import OAuth2AuthorizationCodeBearer
from jose import JOSEError, jwt

from .decorators import returns_errors


TokenPayload = Optional[dict[str, Any]]

logger = logging.getLogger(__file__)

AUTH_BASE_URL = "https://auth.delphai.com/auth/realms/delphai/protocol/openid-connect"

OAuth2Token = OAuth2AuthorizationCodeBearer(
    authorizationUrl=f"{AUTH_BASE_URL}/auth",
    tokenUrl=f"{AUTH_BASE_URL}/token",
    auto_error=False,
)


@Depends
@returns_errors(
    status.HTTP_400_BAD_REQUEST, status.HTTP_401_UNAUTHORIZED, status.HTTP_403_FORBIDDEN
)
async def validated_token(
    token: Annotated[str, Depends(OAuth2Token)],
) -> TokenPayload:
    if not token:
        return {}

    try:
        payload = await decode_token(token)
        payload["_token"] = token
        return payload

    except JOSEError:
        raise HTTPException(status_code=status.HTTP_403_FORBIDDEN)

    except (TypeError, KeyError):
        logger.warning("Wrong token format")
        raise HTTPException(status_code=status.HTTP_400_BAD_REQUEST)


@Depends
class Authorization:
    def __init__(
        self, request: Request, token_payload: Annotated[TokenPayload, validated_token]
    ):
        self._request = request

        self._token_payload = token_payload

        self._client_id = self._scopes = None
        self._user_id = self._user_email = self._user_name = None
        self._groups = self._roles = None

        if self.is_authenticated:
            self._client_id = token_payload["azp"]
            self._scopes = frozenset(token_payload.get("scope").split())

            self._user_id = token_payload["sub"]
            self._user_email = token_payload.get("email")
            self._user_name = token_payload.get("name")
            self._mongo_user_id = token_payload["mongo_user_id"]
            self._mongo_client_id = token_payload["mongo_client_id"]

            self._groups = frozenset(token_payload.get("group_membership", []))
            self._roles = frozenset(
                token_payload.get("realm_access", {}).get("roles", [])
            )

    def require(self, value):
        if not value:
            raise HTTPException(status_code=status.HTTP_403_FORBIDDEN)

    @property
    def is_direct_request(self):
        return not bool(self._request.headers.get("x-forwarded-for"))

    @property
    def is_authenticated(self):
        return bool(self._token_payload)

    @property
    def token_payload(self):
        return self._token_payload

    @property
    def client_id(self):
        return self._client_id

    @property
    def mongo_client_id(self):
        return self._mongo_client_id

    @property
    def scopes(self):
        return self._scopes

    @property
    def user_id(self):
        return self._user_id

    @property
    def mongo_user_id(self):
        return self._mongo_user_id

    @property
    def user_email(self):
        return self._user_email

    @property
    def user_name(self):
        return self._user_name

    @property
    def groups(self):
        return self._groups

    @property
    def roles(self):
        return self._roles

    @property
    def is_api_client(self):
        return self._request.url.hostname.lower().startswith("api.delphai") and (
            "api" in self.roles
        )


trusted_public_keys = {}


async def decode_token(access_token: str) -> str:
    if not trusted_public_keys:
        await _async_fetch_keys()

    decode_args = {"audience": "delphai-gateway", "options": {"leeway": 10}}

    try:
        return jwt.decode(access_token, trusted_public_keys, **decode_args)
    except JOSEError:
        await _async_fetch_keys()
        return jwt.decode(access_token, trusted_public_keys, **decode_args)


http_client = httpx.AsyncClient()


async def _async_fetch_keys():
    global trusted_public_keys

    response = await http_client.get(f"{AUTH_BASE_URL}/certs")
    response.raise_for_status()
    trusted_public_keys = response.json()



from enum import Enum
from bson.objectid import ObjectId as BsonObjectId
from fastapi import Query, Depends
from typing import Tuple


class ObjectId(str):
    @classmethod
    def __get_validators__(cls):
        yield cls.validate

    @classmethod
    def validate(cls, v):
        return str(BsonObjectId(v))

    @classmethod
    def __modify_schema__(cls, field_schema, field):
        field_schema.update(
            type="string",
            minLength=24,
            maxLength=24,
            pattern="^[0-9a-fA-F]{24}$",
        )

        if "example" not in field.field_info.extra:
            field_schema["example"] = field.field_info.extra[
                "example"
            ] = "5ecd2d2d0faf391eadb211a7"


def LimitOffset(limit_default=20, limit_max=300):  # noqa: N802
    @Depends
    def get_limit_offset(
        limit: int = Query(
            default=limit_default,
            ge=0,
            le=limit_max,
            description="The maximum number of entries to be returned per call",
            example=limit_default,
        ),
        offset: int = Query(
            default=0,
            ge=0,
            description="The (zero-based) offset of the first item returned in the collection",  # noqa: E501
            example=0,
        ),
    ) -> Tuple[int, int]:
        return limit, offset

    return get_limit_offset


class Order(Enum):
    asc = "asc"
    desc = "desc"


def Sorting():
    @Depends
    def get_sorting(
        sort_by: str = Query(
            default="",
            description="Sorting field",
            example="name",
        ),
        order_by: Order = Query(
            default="asc",
            description="Sorting order",
            example="asc",
        ),
    ) -> Tuple[str, str]:
        return sort_by, order_by

    return get_sorting



from typing import Dict, List, Optional

from fastapi_camelcase import CamelModel
from pydantic import Field

from ..types import ObjectId
from ..models import Location, Source


class EmployeeCount(CamelModel):
    min: Optional[int] = Field(
        description="Bottom range of the employee count interval", example=11
    )
    max: Optional[int] = Field(
        description="Top range of the employee count interval", example=50
    )
    exact: Optional[int] = Field(description="Exact number for employees", example=30)
    range: Optional[str] = Field(
        description="Employee count interval displayed in delphai", example="11-50"
    )


class CompanyDescription(CamelModel):
    long: Optional[str] = Field(
        description="Company's default description",
        example=(
            "delphai is an AI and big data analytics software platform that informs "
            "business decisions and validates strategies"
        ),
    )
    short: Optional[str] = Field(
        description="Truncated version of company's default description",
        example="delphai is an AI and big data analytics software platform",
    )


class CompanyRevenue(CamelModel):
    currency: Optional[str] = Field(
        description="Currency of revenue number", example="EUR"
    )
    annual: Optional[int] = Field(
        description="Annual revenue number for specified year", example=5000000
    )
    year: Optional[int] = Field(description="Year of revenue number", example=2022)
    source: Source


class Company(CamelModel):
    id: ObjectId = Field(..., description="Internal company ID")
    name: Optional[str] = Field(description="Name of the company", example="delphai")
    url: str = Field(..., description="Webpage of the company", example="delphai.com")
    company_href: str = Field(
        ..., description="Full webpage link", example="http://delphai.com"
    )
    descriptions: Optional[Dict[str, CompanyDescription]]
    founding_year: Optional[int] = Field(description="Founding year", example=2020)
    headquarters: Optional[Location] = Field(description="Company address")
    employee_count: Optional[EmployeeCount] = Field(description="Number of employees")
    additional_urls: Optional[Dict[str, str]] = Field(
        example={"linkedin": "https://www.linkedin.com/company/delphai"}
    )
    revenue: Optional[Dict[str, CompanyRevenue]] = Field(
        description="Company revenue with currency"
    )
    products: Optional[List[str]] = Field(
        description="List of company products", example=["Software"]
    )


class ProjectsCompany(CamelModel):
    company: Company
    custom_labels: Optional[List[str]] = Field(
        description="Custom labels assigned by user"
    )
    assigned_by: str = Field(description="By whom the company was added to the project")
    disabled: bool = Field(description="If the company is available on delphai")


class CompaniesSearchResult(CamelModel):
    company: Company
    score: float = Field(default=0, description="Search score", example="202.35745")
    snippets: List[str] = Field(
        default=[],
        description="Snippets containing query keywords",
        example=[
            "delphai is an AI and big data analytics software platform that informs "
            "business decisions and validates strategies"
        ],
    )


class CompaniesSearchResults(CamelModel):
    results: List[CompaniesSearchResult]
    total: int = Field(..., description="Number of results", example=1337)


class CompanyPeer(CamelModel):
    company: Company
    score: float = Field(default=0, description="Search score", example="202.35745")


class CompanyPeers(CamelModel):
    results: List[CompanyPeer]
    total: int = Field(..., description="Number of results", example=5)



from typing import List, Optional

from fastapi_camelcase import CamelModel
from pydantic import Field
from datetime import datetime


class HTTPExceptionModel(CamelModel):
    detail: str


class Label(CamelModel):
    name: str = Field(description="Assigned label")
    children: List["Label"] = Field(description="Sublabels")


class Location(CamelModel):
    country: Optional[str] = Field(
        description="Company address (country)", example="Germany"
    )
    city: Optional[str] = Field(description="Company address (city)", example="Berlin")
    continent: Optional[str] = Field(
        description="Company address (continent)", example="Europe"
    )
    state: Optional[str] = Field(
        description="Company address (state/land)", example="Berlin"
    )
    latitude: Optional[float] = Field(example=52.5167)
    longitude: Optional[float] = Field(example=13.3833)
    zip_code: Optional[str] = Field(
        description="Company address (ZIP code)", example="10999"
    )


class Project(CamelModel):
    total: int = Field(
        ..., description="Total number of companies in this project", example=35
    )
    name: str = Field(
        ..., description="Name of the project", example="Healthcare | Startups"
    )
    created: datetime = Field(..., description="When the project was created")
    last_modified: datetime = Field(
        ..., description="When the project was last edited or updated"
    )
    available: int = Field(
        ...,
        description="Number of companies in this project that are accessible in the dashboard",
        example=29,
    )


class Source(CamelModel):
    name: str = Field(description="Name of the source")
    credibility_score: float = Field(
        description="Credibility score of source in percentage", example=0.60
    )



from .models import HTTPExceptionModel


def returns_errors(*error_codes):
    responses = {
        str(error_code): {"model": HTTPExceptionModel} for error_code in error_codes
    }

    def decorator(func):
        original_responses = getattr(func, "responses", {})
        func.responses = dict(original_responses, **responses)
        return func

    return decorator



from datetime import datetime
from enum import Enum
from fastapi_camelcase import CamelModel
from pydantic import Field
from typing import List, Optional

from ..models import Label
from ..types import ObjectId


class NewsArticleType(str, Enum):
    NEWS = "news"
    PRESS_RELEASE = "press release"


class NewsArticle(CamelModel):
    company_id: ObjectId = Field(..., description="Internal company ID")
    url: str = Field(..., description="Article URL")
    type: NewsArticleType = Field(..., description="Type of article")
    published: datetime = Field(..., description="When the article was published")
    snippet: str = Field(
        ..., description="Snippet of the article mentioning the company"
    )
    language: Optional[str] = Field(
        description="Original language of the article in ISO 639 code"
    )
    labels: Optional[List[Label]]
    title: str = Field(..., description="Article title")
    added: datetime = Field(..., description="When the article was added to delphai")


class NewsArticles(CamelModel):
    results: List[NewsArticle]
    total: int = Field(..., description="Number of results")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['delphai_fastapi',
 'delphai_fastapi.companies',
 'delphai_fastapi.job_posts',
 'delphai_fastapi.news_articles']

package_data = \
{'': ['*']}

install_requires = \
['fastapi-camelcase>=1,<2',
 'fastapi>=0,<1',
 'httpx>=0,<1',
 'pymongo>3',
 'python-jose>=3.3,<4.0']

setup_kwargs = {
    'name': 'delphai-fastapi',
    'version': '0.1.11',
    'description': 'Package for fastAPI models',
    'long_description': 'None',
    'author': 'Berinike Tech',
    'author_email': 'berinike@delphai.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



from datetime import datetime
from fastapi_camelcase import CamelModel
from pydantic import Field
from typing import List, Optional

from ..types import ObjectId


class JobPost(CamelModel):
    company_id: ObjectId = Field(..., description="Internal company ID")
    url: str = Field(..., description="Job post URL")
    published: datetime = Field(..., description="When the job post was published")
    location: Optional[str] = Field(description="Location of the position")
    job_description: Optional[str] = Field(description="Description of the position")
    language: Optional[str] = Field(description="Original language of the job post")
    title: str = Field(..., description="Position title")
    added: datetime = Field(..., description="When the job post was added to delphai")


class JobPosts(CamelModel):
    results: List[JobPost]
    total: int = Field(..., description="Number of results")
